# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: perc.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import builder as _builder
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from . import coord_pb2 as coord__pb2
from . import uuid_pb2 as uuid__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\nperc.proto\x1a\x0b\x63oord.proto\x1a\nuuid.proto\"]\n\rPercPersonFer\x12\x0f\n\x07neutral\x18\x01 \x01(\x02\x12\r\n\x05happy\x18\x02 \x01(\x02\x12\x0b\n\x03sad\x18\x03 \x01(\x02\x12\x10\n\x08surprise\x18\x04 \x01(\x02\x12\r\n\x05\x61nger\x18\x05 \x01(\x02\"\x98\x01\n\x0cPercPersonLm\x12\x19\n\x05\x65ye_l\x18\x01 \x01(\x0b\x32\n.CPoint2dS\x12\x19\n\x05\x65ye_r\x18\x02 \x01(\x0b\x32\n.CPoint2dS\x12\x18\n\x04nose\x18\x03 \x01(\x0b\x32\n.CPoint2dS\x12\x1b\n\x07mouth_l\x18\x04 \x01(\x0b\x32\n.CPoint2dS\x12\x1b\n\x07mouth_r\x18\x05 \x01(\x0b\x32\n.CPoint2dS\"\xf3\x02\n\nPercPerson\x12\x10\n\x08id_score\x18\x02 \x01(\x02\x12\x13\n\x04uuid\x18\x0f \x01(\x0b\x32\x05.Uuid\x12\x17\n\x04\x66\x61\x63\x65\x18\x03 \x01(\x0b\x32\t.CBbox2dS\x12\x19\n\x02lm\x18\x04 \x01(\x0b\x32\r.PercPersonLm\x12\x14\n\x02hp\x18\x05 \x01(\x0b\x32\x08.CBryanS\x12\x1a\n\x04gaze\x18\x06 \x01(\x0b\x32\x0c.CCartVec3dS\x12\x1b\n\x03\x66\x65r\x18\x07 \x01(\x0b\x32\x0e.PercPersonFer\x12\x0f\n\x07\x64ist_mm\x18\x08 \x01(\x02\x12\x14\n\x0cgaze_overlap\x18\t \x01(\x02\x12\x1d\n\x07g_eye_l\x18\n \x03(\x0b\x32\x0c.CCartSys3dS\x12\x1d\n\x07g_eye_r\x18\x0b \x03(\x0b\x32\x0c.CCartSys3dS\x12\x1c\n\x06g_nose\x18\x0c \x03(\x0b\x32\x0c.CCartSys3dS\x12\x1a\n\x04g_hp\x18\r \x03(\x0b\x32\x0c.CCartSys3dS\x12\x1c\n\x06g_gaze\x18\x0e \x03(\x0b\x32\x0c.CCartSys3dS\"\'\n\nPercObject\x12\x19\n\x03loc\x18\x01 \x01(\x0b\x32\x0c.CCartVec3dS\"Z\n\x0bPercSstMeta\x12\n\n\x02id\x18\x01 \x01(\x04\x12\x10\n\x08\x61\x63tivity\x18\x02 \x01(\x02\x12\x19\n\x03loc\x18\x03 \x01(\x0b\x32\x0c.CCartSys3dS\x12\x12\n\nis_dynamic\x18\x04 \x01(\x08\"\x9d\x01\n\x08PercData\x12\x0c\n\x04time\x18\x01 \x01(\x04\x12\r\n\x05\x66lags\x18\x02 \x01(\r\x12\x19\n\x04pers\x18\x03 \x03(\x0b\x32\x0b.PercPerson\x12\x18\n\x03obj\x18\x04 \x03(\x0b\x32\x0b.PercObject\x12\x17\n\x0fsst_time_latest\x18\x05 \x01(\x04\x12&\n\x10sst_track_latest\x18\x06 \x03(\x0b\x32\x0c.PercSstMetaB\nZ\x08./;protob\x06proto3')

_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, globals())
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'perc_pb2', globals())
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'Z\010./;proto'
  _PERCPERSONFER._serialized_start=39
  _PERCPERSONFER._serialized_end=132
  _PERCPERSONLM._serialized_start=135
  _PERCPERSONLM._serialized_end=287
  _PERCPERSON._serialized_start=290
  _PERCPERSON._serialized_end=661
  _PERCOBJECT._serialized_start=663
  _PERCOBJECT._serialized_end=702
  _PERCSSTMETA._serialized_start=704
  _PERCSSTMETA._serialized_end=794
  _PERCDATA._serialized_start=797
  _PERCDATA._serialized_end=954
# @@protoc_insertion_point(module_scope)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: uuid.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import builder as _builder
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\nuuid.proto\"6\n\x04Uuid\x12\n\n\x02\x63\x30\x18\x01 \x01(\x07\x12\n\n\x02\x63\x31\x18\x02 \x01(\x07\x12\n\n\x02\x63\x32\x18\x03 \x01(\x07\x12\n\n\x02\x63\x33\x18\x04 \x01(\x07\x62\x06proto3')

_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, globals())
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'uuid_pb2', globals())
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  _UUID._serialized_start=14
  _UUID._serialized_end=68
# @@protoc_insertion_point(module_scope)



from navel._data_structs import (
    Bbox2d,
    Bryan,
    CartSys3d,
    CartVec3d,
    CoordSystem,
    DataType,
    PerceptionData,
    Person,
    PersonFacialExpression,
    PersonLandmarks,
    Point2d,
    SstMeta,
)
from navel._interface import Motion, Perception, Robot, Speech, run
from navel._version import __version__



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: coord.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import builder as _builder
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x0b\x63oord.proto\".\n\x0b\x43\x43\x61rtVec3dS\x12\t\n\x01x\x18\x01 \x01(\x02\x12\t\n\x01y\x18\x02 \x01(\x02\x12\t\n\x01z\x18\x03 \x01(\x02\":\n\x08\x43\x42\x62ox2dS\x12\n\n\x02x1\x18\x01 \x01(\x05\x12\n\n\x02y1\x18\x02 \x01(\x05\x12\n\n\x02x2\x18\x03 \x01(\x05\x12\n\n\x02y2\x18\x04 \x01(\x05\"!\n\tCPoint2dS\x12\t\n\x01x\x18\x01 \x01(\x05\x12\t\n\x01y\x18\x02 \x01(\x05\"*\n\x07\x43\x42ryanS\x12\t\n\x01x\x18\x01 \x01(\x02\x12\t\n\x01y\x18\x02 \x01(\x02\x12\t\n\x01z\x18\x03 \x01(\x02\"I\n\x0b\x43\x43\x61rtSys3dS\x12\x19\n\x03sys\x18\x01 \x01(\x0e\x32\x0c.CoordSystem\x12\t\n\x01x\x18\x02 \x01(\x02\x12\t\n\x01y\x18\x03 \x01(\x02\x12\t\n\x01z\x18\x04 \x01(\x02*\x95\x01\n\x0b\x43oordSystem\x12\r\n\tUNDEFINED\x10\x00\x12\r\n\tCAM_CHEST\x10\x01\x12\x0c\n\x08HEAD_REF\x10\x02\x12\x11\n\rHEAD_STRAIGHT\x10\x03\x12\r\n\tHEAD_REAL\x10\x04\x12\x0c\n\x08\x43\x41M_HEAD\x10\x05\x12\t\n\x05\x45YE_L\x10\x06\x12\t\n\x05\x45YE_R\x10\x07\x12\x14\n\x10\x43OORD_SYSTEM_LEN\x10\x08\x42\nZ\x08./;protob\x06proto3')

_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, globals())
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'coord_pb2', globals())
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'Z\010./;proto'
  _COORDSYSTEM._serialized_start=278
  _COORDSYSTEM._serialized_end=427
  _CCARTVEC3DS._serialized_start=15
  _CCARTVEC3DS._serialized_end=61
  _CBBOX2DS._serialized_start=63
  _CBBOX2DS._serialized_end=121
  _CPOINT2DS._serialized_start=123
  _CPOINT2DS._serialized_end=156
  _CBRYANS._serialized_start=158
  _CBRYANS._serialized_end=200
  _CCARTSYS3DS._serialized_start=202
  _CCARTSYS3DS._serialized_end=275
# @@protoc_insertion_point(module_scope)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: config.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import builder as _builder
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x0c\x63onfig.proto\"\xe0\x03\n\x12\x43onfigGetDescEntry\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\x19\n\x04type\x18\x02 \x01(\x0e\x32\x0b.ConfigType\x12\x11\n\x07min_i32\x18\x03 \x01(\x05H\x00\x12\x11\n\x07min_u32\x18\x04 \x01(\rH\x00\x12\x11\n\x07min_i64\x18\x05 \x01(\x03H\x00\x12\x11\n\x07min_u64\x18\x06 \x01(\x04H\x00\x12\x11\n\x07min_f32\x18\x07 \x01(\x02H\x00\x12\x11\n\x07min_f64\x18\x08 \x01(\x01H\x00\x12\x11\n\x07max_i32\x18\t \x01(\x05H\x01\x12\x11\n\x07max_u32\x18\n \x01(\rH\x01\x12\x11\n\x07max_i64\x18\x0b \x01(\x03H\x01\x12\x11\n\x07max_u64\x18\x0c \x01(\x04H\x01\x12\x11\n\x07max_f32\x18\r \x01(\x02H\x01\x12\x11\n\x07max_f64\x18\x0e \x01(\x01H\x01\x12\x15\n\x0b\x64\x65\x66_val_i32\x18\x0f \x01(\x05H\x02\x12\x15\n\x0b\x64\x65\x66_val_u32\x18\x10 \x01(\rH\x02\x12\x15\n\x0b\x64\x65\x66_val_i64\x18\x11 \x01(\x03H\x02\x12\x15\n\x0b\x64\x65\x66_val_u64\x18\x12 \x01(\x04H\x02\x12\x15\n\x0b\x64\x65\x66_val_f32\x18\x13 \x01(\x02H\x02\x12\x15\n\x0b\x64\x65\x66_val_f64\x18\x14 \x01(\x01H\x02\x12\r\n\x05\x66lags\x18\x15 \x01(\r\x12\x0b\n\x03len\x18\x16 \x01(\rB\x05\n\x03minB\x05\n\x03maxB\t\n\x07\x64\x65\x66_val\"1\n\rConfigGetDesc\x12 \n\x03\x63\x66g\x18\x01 \x03(\x0b\x32\x13.ConfigGetDescEntry\"\x1e\n\nConfigSave\x12\x10\n\x08\x66ilename\x18\x01 \x01(\t*U\n\x10\x43onfigTypeOffset\x12\r\n\tundefined\x10\x00\x12\x0e\n\nmin_offset\x10\x03\x12\x0e\n\nmax_offset\x10\t\x12\x12\n\x0e\x64\x65\x66_val_offset\x10\x0f*Y\n\nConfigType\x12\x0b\n\x07int32_t\x10\x00\x12\x0c\n\x08uint32_t\x10\x01\x12\x0b\n\x07int64_t\x10\x02\x12\x0c\n\x08uint64_t\x10\x03\x12\t\n\x05\x66loat\x10\x04\x12\n\n\x06\x64ouble\x10\x05\x42\nZ\x08./;protob\x06proto3')

_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, globals())
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'config_pb2', globals())
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'Z\010./;proto'
  _CONFIGTYPEOFFSET._serialized_start=582
  _CONFIGTYPEOFFSET._serialized_end=667
  _CONFIGTYPE._serialized_start=669
  _CONFIGTYPE._serialized_end=758
  _CONFIGGETDESCENTRY._serialized_start=17
  _CONFIGGETDESCENTRY._serialized_end=497
  _CONFIGGETDESC._serialized_start=499
  _CONFIGGETDESC._serialized_end=548
  _CONFIGSAVE._serialized_start=550
  _CONFIGSAVE._serialized_end=580
# @@protoc_insertion_point(module_scope)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: payload.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import builder as _builder
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from . import coord_pb2 as coord__pb2
from . import uuid_pb2 as uuid__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\rpayload.proto\x1a\x0b\x63oord.proto\x1a\nuuid.proto\"\x15\n\x06Strmsg\x12\x0b\n\x03str\x18\x01 \x01(\t\"6\n\nLookatCart\x12\x1a\n\x04\x63\x61rt\x18\x01 \x01(\x0b\x32\x0c.CCartSys3dS\x12\x0c\n\x04head\x18\x02 \x01(\x02\"0\n\x08LookatPx\x12\x16\n\x02px\x18\x01 \x01(\x0b\x32\n.CPoint2dS\x12\x0c\n\x04head\x18\x02 \x01(\x02\"1\n\x0cLookatPerson\x12\x13\n\x04uuid\x18\x01 \x01(\x0b\x32\x05.Uuid\x12\x0c\n\x04head\x18\x02 \x01(\x02\"8\n\tHeadBndry\x12\r\n\x05v_max\x18\x01 \x01(\x02\x12\r\n\x05\x61_max\x18\x02 \x01(\x02\x12\r\n\x05j_max\x18\x03 \x01(\x02\"(\n\x0bHeadOverlay\x12\x19\n\x07overlay\x18\x01 \x01(\x0b\x32\x08.CBryanS\"e\n\x06HeadFe\x12\x0f\n\x07neutral\x18\x01 \x01(\x02\x12\r\n\x05happy\x18\x02 \x01(\x02\x12\x0b\n\x03sad\x18\x03 \x01(\x02\x12\x10\n\x08surprise\x18\x04 \x01(\x02\x12\r\n\x05\x61nger\x18\x05 \x01(\x02\x12\r\n\x05smile\x18\x06 \x01(\x02\"5\n\x0bHeadEyelids\x12\x12\n\nopenness_l\x18\x01 \x01(\x02\x12\x12\n\nopenness_r\x18\x02 \x01(\x02\"\x1d\n\tHeadMouth\x12\x10\n\x08openness\x18\x01 \x01(\x02\"|\n\tConfigSet\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\r\n\x03i32\x18\x02 \x01(\x05H\x00\x12\r\n\x03u32\x18\x03 \x01(\rH\x00\x12\r\n\x03i64\x18\x04 \x01(\x03H\x00\x12\r\n\x03u64\x18\x05 \x01(\x04H\x00\x12\r\n\x03\x66\x33\x32\x18\x06 \x01(\x02H\x00\x12\r\n\x03\x66\x36\x34\x18\x07 \x01(\x01H\x00\x42\x07\n\x05value\"*\n\x07\x42\x61seinc\x12\t\n\x01x\x18\x01 \x01(\x05\x12\t\n\x01y\x18\x02 \x01(\x05\x12\t\n\x01z\x18\x03 \x01(\x05\" \n\x08\x42\x61setilt\x12\t\n\x01x\x18\x01 \x01(\x05\x12\t\n\x01y\x18\x02 \x01(\x05\"(\n\x04\x41rms\x12\x0f\n\x07\x61ngle_l\x18\x01 \x01(\x05\x12\x0f\n\x07\x61ngle_r\x18\x02 \x01(\x05\x42\nZ\x08./;protob\x06proto3')

_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, globals())
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'payload_pb2', globals())
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'Z\010./;proto'
  _STRMSG._serialized_start=42
  _STRMSG._serialized_end=63
  _LOOKATCART._serialized_start=65
  _LOOKATCART._serialized_end=119
  _LOOKATPX._serialized_start=121
  _LOOKATPX._serialized_end=169
  _LOOKATPERSON._serialized_start=171
  _LOOKATPERSON._serialized_end=220
  _HEADBNDRY._serialized_start=222
  _HEADBNDRY._serialized_end=278
  _HEADOVERLAY._serialized_start=280
  _HEADOVERLAY._serialized_end=320
  _HEADFE._serialized_start=322
  _HEADFE._serialized_end=423
  _HEADEYELIDS._serialized_start=425
  _HEADEYELIDS._serialized_end=478
  _HEADMOUTH._serialized_start=480
  _HEADMOUTH._serialized_end=509
  _CONFIGSET._serialized_start=511
  _CONFIGSET._serialized_end=635
  _BASEINC._serialized_start=637
  _BASEINC._serialized_end=679
  _BASETILT._serialized_start=681
  _BASETILT._serialized_end=713
  _ARMS._serialized_start=715
  _ARMS._serialized_end=755
# @@protoc_insertion_point(module_scope)



from __future__ import annotations, print_function

import errno
import socket

from navel._pyproto import klframe_pb2 as _klframe

# Buffer size of socket; warning required size scales linearly with people
# detected
_SC_SOCK_BUF = 1024 * 10


class ProtobufSocket(object):
    """Creates an object which can transceive protobuf messages via UDS.

    Args:
        socket_name (str): Name of socket to connect to.
    """

    def __init__(self, socket_name):
        self._sock = 0
        self._socket_name = socket_name

        self.connect()

    def __del__(self):
        self.disconnect()

    def connect(self, attempts=1):
        """Connects object to its Unix domain socket.

        Will attempt to connect to UDS set at object creation. If the
        attempts argument is passed and connection fails, this method will
        loop multiple times to reconnect.

        Args:
            attempts (int, optional): How many attempts should be made to
                connect. Defaults to 1.
        Raises:
            ConnectionError: If unable to connect.
        """

        self._sock = socket.socket(socket.AF_UNIX, socket.SOCK_SEQPACKET)

        for i in range(attempts):
            try:
                self._sock.connect(self._socket_name)
                return

            except OSError as err:
                if err.errno == errno.EISCONN:
                    return
                if i == attempts - 1:
                    msg = f"Failed to connect to {self._socket_name}"
                    raise ConnectionError(msg) from err

    def disconnect(self):
        """Disconnects socket."""

        self._sock.close()

    def send(self, data, ptype):
        msg = _klframe.Klframe()
        msg.data = data.SerializeToString()
        msg.ptype = ptype
        msg = msg.SerializeToString()

        self._sock.sendall(msg)

    def recv(self):
        data = self._sock.recv(_SC_SOCK_BUF, socket.MSG_DONTWAIT)

        if data:
            msg = _klframe.Klframe()
            msg.ParseFromString(data)
            return msg



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: sys.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import builder as _builder
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\tsys.proto\"u\n\x0bSystemState\x12\x10\n\x08\x61gx_mode\x18\x01 \x01(\r\x12\x11\n\tbattery_u\x18\x02 \x01(\x05\x12\x15\n\rbattery_chg_i\x18\x03 \x01(\x05\x12\x13\n\x0bis_charging\x18\x04 \x01(\x08\x12\x15\n\rbattery_level\x18\x05 \x01(\x02\x62\x06proto3')

_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, globals())
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'sys_pb2', globals())
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  _SYSTEMSTATE._serialized_start=13
  _SYSTEMSTATE._serialized_end=130
# @@protoc_insertion_point(module_scope)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: klframe.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import builder as _builder
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\rklframe.proto\".\n\x07Klframe\x12\x15\n\x05ptype\x18\x01 \x01(\x0e\x32\x06.Ptype\x12\x0c\n\x04\x64\x61ta\x18\x02 \x01(\x0c*\x91\x03\n\x05Ptype\x12\n\n\x06STRMSG\x10\x00\x12\x08\n\x04PERC\x10\x01\x12\x0f\n\x0bLOOKAT_CART\x10\x04\x12\r\n\tLOOKAT_PX\x10\x05\x12\x11\n\rLOOKAT_PERSON\x10\x06\x12\x10\n\x0cHEAD_OVERLAY\x10\x07\x12\x0b\n\x07HEAD_FE\x10\x08\x12\x10\n\x0cHEAD_EYELIDS\x10\t\x12\x0e\n\nHEAD_MOUTH\x10\n\x12\x0e\n\nCONFIG_SET\x10\x0b\x12\x13\n\x0f\x43ONFIG_GET_DESC\x10\x0c\x12\x0e\n\nCONFIG_GET\x10\r\x12\x0f\n\x0b\x43ONFIG_SAVE\x10\x0e\x12\x0f\n\x0bSPEECH_TEXT\x10\x0f\x12\x14\n\x10SPEECH_ASR_STATE\x10\x10\x12\x0b\n\x07\x42\x41SEINC\x10\x11\x12\x0c\n\x08\x42\x41SETILT\x10\x12\x12\x08\n\x04\x41RMS\x10\x13\x12\x13\n\x0fSPEECH_BOOKMARK\x10\x14\x12\x0e\n\nHEAD_BNDRY\x10\x15\x12\x10\n\x0cSYSTEM_STATE\x10\x16\x12\r\n\tPTYPE_LEN\x10\x17\"\x04\x08\x02\x10\x02\"\x04\x08\x03\x10\x03*\rNLDRIVE_STATE*\x0bNLDRIVE_CMDB\nZ\x08./;protob\x06proto3')

_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, globals())
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'klframe_pb2', globals())
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'Z\010./;proto'
  _PTYPE._serialized_start=66
  _PTYPE._serialized_end=467
  _KLFRAME._serialized_start=17
  _KLFRAME._serialized_end=63
# @@protoc_insertion_point(module_scope)



import asyncio
from warnings import warn

import alsaaudio

from navel import _messages
from navel._async import AsyncRunner

NAVEL_MOTION_SOCKET = "/run/bodyd/motion.sock"
NAVEL_PERCEPTION_SOCKET = "/run/bodyd/vision.sock"
NAVEL_SPEECH_SOCKET = "/run/bodyd/speech.sock"

MVC1_VOL_MAX = 16000  # Tegra specific max volume
MVC1_VOL_LIMIT = 0.8  # Relative to MVC1_VOL_MAX


class Motion(_messages.MotionMessenger):
    """Send messages to motion socket.

    This class is able to send protobuf messages via a unix
    domain socket (UDS). As soon as the object is created it will
    attempt to connect to the socket. If socket connection is successful,
    methods can be used to send messages. Each message is  converted
    to a protobuf format before being sent to the motion socket.

    Args:
        socket_name (str): Name of motion socket to connect too.
    """

    def __init__(self, socket_name=NAVEL_MOTION_SOCKET):
        super().__init__(socket_name)


class Speech(_messages.SpeechMessenger):
    """Send and receive messages via speech socket

    This class is able to transceive protobuf messages via a unix
    domain socket (UDS). As soon as the object is created it will
    attempt to connect to the socket. If socket connection is successful,
    methods can be used to send messages. Each message is converted
    into a protobuf format before being sent to the speech socket.

    It also provides a mechanism for registerig bookmarks in the text.
    This is run asynchronously as messages could arrive at any point in
    time. Read messages are accessed using callback functions.
    If there are no relevent callback functions then messages will be thrown
    away to keep buffer clear. See valid keys to see what callbacks are
    supported.

    If the object where data is stored by callback functions is to be accessed
    by any other threads, steps must be taken to make it threadsafe.

    Args:
        socket_name (str, optional): Name of the socket to connect to.
            Defaults to NAVEL_SPEECH_SOCKET
    """

    def __init__(self, socket_name=NAVEL_SPEECH_SOCKET):
        super().__init__(socket_name)


class Perception(_messages.PerceptionMessenger):
    """Receive messages from perception socket

    This class is able to receive protobuf messages via a unix
    domain socket (UDS). As soon as the object is created it will
    attempt to connect to the socket. If socket connection is successful,
    methods can be used to receive messages. Each message is converted
    from a protobuf format and into a navel data struct.

    Args:
        socket_name (str, optional): Name of the socket to connect to.
            Defaults to NAVEL_PERCEPTION_SOCKET
    """

    def __init__(self, socket_name=NAVEL_PERCEPTION_SOCKET):
        super().__init__(socket_name)


class Robot(Motion, Perception, Speech):
    """Send commands and receive data from robot.

    This class can be used to send motion or speech commands, register
    callbacks for bookmarks, fetch the latest perception data, and set
    robot configuration values. It should always be used as a context
    manager (inside a ``with`` block).
    """

    def __init__(self):
        Motion.__init__(self, None)
        Perception.__init__(self, None)
        Speech.__init__(self, None)
        self._sockets = {}
        self._mixer = None

    def open(self):
        AsyncRunner.open(self)

    def close(self):
        for socket in self._sockets.values():
            socket.disconnect()

        self._sockets = {}
        AsyncRunner.close(self)

    @property
    def _motion_sock(self):
        return self._get_or_init_prop(NAVEL_MOTION_SOCKET)

    @property
    def _perc_sock(self):
        return self._get_or_init_prop(NAVEL_PERCEPTION_SOCKET)

    @property
    def _speech_sock(self):
        return self._get_or_init_prop(NAVEL_SPEECH_SOCKET)

    def _get_or_init_prop(self, name):
        if not self._active:
            raise RuntimeError(
                'Robot communication should only be done inside "with" block'
            )
        if name not in self._sockets:
            self._sockets[name] = _messages.ProtobufSocket(name)
        return self._sockets[name]

    @property
    def mixer(self) -> alsaaudio.Mixer:
        if self._mixer is None:
            self._mixer = alsaaudio.Mixer(control="MVC1", cardindex=0)

        return self._mixer

    @property
    def volume(self) -> int:
        """The current volume of the robot speakers."""

        vol_max = MVC1_VOL_LIMIT * MVC1_VOL_MAX
        vol = self.mixer.getvolume(units=alsaaudio.VOLUME_UNITS_RAW)[0]
        vol = round((vol / vol_max) ** (1 / 0.15) * 100)

        return vol

    @volume.setter
    def volume(self, value: int) -> None:
        if not 0 <= value <= 100:
            raise ValueError("Only volumes between 0 and 100 are supported")

        vol_max = MVC1_VOL_LIMIT * MVC1_VOL_MAX
        vol = round((value / 100) ** 0.15 * vol_max)

        self.mixer.setvolume(vol, units=alsaaudio.VOLUME_UNITS_RAW)


def run(app):
    """Run a coroutine with a robot connection as the only parameter.

    This is used to start navel "apps". This function is only provided
    as a shorthand for the most common use-case, calling the run
    method on any of the connection classes accomplishes the same goal.
    """
    warn(
        "Function .run() is deprecated and will be removed in a future version. Please check the getting started guide for the recommended way to start coroutines.",
        DeprecationWarning,
        stacklevel=2,
    )

    async def wrapped():
        async with Robot() as r:
            return await app(r)

    return asyncio.run(wrapped())



from dataclasses import dataclass, field
from enum import IntEnum
from typing import List


class DataType(IntEnum):
    """Enum class describing what data type is used."""

    I32 = 1  #: 32 bit integer
    U32 = 2  #: 32 bit unsigned integer
    I64 = 3  #: 64 bit integer
    U64 = 4  #: 64 bit unsigned integer
    F32 = 5  #: 32 bit float
    F64 = 6  #: 64 bit double


class CoordSystem(IntEnum):
    """Enum class describing available coordinate frames. X generally points
    forward, Y to the left and Z upwards (from point of view of robot).
    """

    CAM_CHEST = 1  #: Origin in center of chest cam, not implemented yet
    HEAD_REF = 2  #: Origin in head center, head facing slightly downwards.
    HEAD_STRAIGHT = 3  #: Origin in head center, head facing straight.
    HEAD_REAL = 4  #: Fixed to head, x axis pointing in facing direction.
    CAM_HEAD = 5  #: Origin in center of head cam.
    EYE_LEFT = 6  #: Origin in center of left eye.
    EYE_RIGHT = 7  #: Origin in center of right eye.
    COORD_LEN = 8  #: Max Value of enum, used only in calculations


@dataclass
class CartVec3d(object):
    """Describes a cartesian 3D vector.

    Args:
            x (float): X vector.
            y (float): Y vector.
            z (float): Z vector.
    """

    x: float
    y: float
    z: float


@dataclass
class Bbox2d(object):
    """Describes a bounding box with two 2D points.

    Sets the limits of a bounding Box located in an image. The bounding
    box limits are described in pixels in XY plane.

    Args:
        x1 (int): Point 1 X coordinate in pixels.
        y1 (int): Point 1 Y coordinate in pixels.
        x2 (int): Point 2 X coordinate in pixels.
        y2 (int): Point 2 Y coordinate in pixels.
    """

    x1: int
    y1: int
    x2: int
    y2: int


@dataclass
class Point2d(object):
    """Describes a 2D point using pixel coordinates.

    Args:
        x (int): X coordinate in pixels.
        y (int): Y coordinate in pixels.
    """

    x: int
    y: int


@dataclass
class Bryan(object):
    """Describes a Tait-Bryan angle rotation.

    Args:
        x (float): Rotation around X axis in radians.
        y (float): Rotation around Y axis in radians.
        z (float): Rotation around Z axis in radians.
    """

    x: float
    y: float
    z: float


@dataclass
class CartSys3d(object):
    """Describes a 3D point in cartesian space.

    Args:
        sys (CoordSystem): Coordinate system used.
        x (float): X coordinate in meters.
        y (float): Y coordinate in meters.
        z (float): Z coordinate in meters.
    """

    sys: CoordSystem
    x: float
    y: float
    z: float


@dataclass
class PersonFacialExpression(object):
    """Class describing the facial expression of the detected person

    Each emotion in this class is described by a magnitude where 1
    describes maximum expression and 0 describes minimum expression.

    Args:
        neutral (float): magnitude of neutral.
        happy (float): magnitude of happyness.
        sad (float): magnitude of sadness.
        surprise (float): magnitude of surprise.
        anger (float): magnitude of anger.
    """

    neutral: float
    happy: float
    sad: float
    surprise: float
    anger: float


@dataclass
class PersonLandmarks(object):
    """Class describing position of a person's facial landmarks.

    Each landmark is described by its 2D position within the host image.

    Args:
        eye_left (Point2d): Position of left eye.
        eye_right (Point2d): Position of right eye.
        nose (Point2d): Position of nose.
        mouth_left (Point2d): Position of left end of mouth.
        mouth_right (Point2d): Position of right end of mouth.
    """

    eye_left: Point2d
    eye_right: Point2d
    nose: Point2d
    mouth_left: Point2d
    mouth_right: Point2d


@dataclass
class Person(object):
    """Defines attributes of a detected person.

    Args:
        id_score (float):
            Defaults to None.
        user_id (float): Unique user id.
            Defaults to None.
        face (Bbox2d): Bounding box of face.
            Defaults to None.
        landmarks (PersonLandmarks): Position of left end of mouth.
            Defaults to None.
        head_position (Bryan): Position of head in Tait-Bryan angles (radians).
            Defaults to None.
        gaze (CartVec3d): Vector describing persons direction of gaze.
            Defaults to None.
        facial_expression (PersonFacialExpression): Person's facial expression.
            Defaults to None.
        dist_mm (float): Distance from robot to person (mm).
            Defaults to None.
        gaze_overlap (float): How much person is staring at robot, 0 = not
            looking at robot, 1 = looking directly at robot.
            Defaults to None.
        g_eye_left (CartSys3d): A list of the position of left eye in all coord
            frames. This might be out of frame, then sys is CoordSystem.UNDEFINED.
            Defaults to None.
        g_eye_right (CartSys3d): A list of the position of right eye in all
            coord frames. This might be out of frame, then sys is
            CoordSystem.UNDEFINED.
            Defaults to None.
        g_nose (CartSys3d): A list of the position of nose in all
            coord frames. This might be out of frame, then sys is
            CoordSystem.UNDEFINED.
            Defaults to None.
        g_head_position (CartSys3d): A list of the position of head in all
            coord frames.
            Defaults to None.
        g_gaze (CartSys3d): A list of the target of persons gaze in all
            coord frames.
            Defaults to None.
    """

    id_score: float | None = None
    uuid: float | None = None
    face: Bbox2d | None = None
    landmarks: PersonLandmarks | None = None
    head_position: Bryan | None = None
    gaze: CartVec3d | None = None
    facial_expression: PersonFacialExpression | None = None
    dist_mm: float | None = None
    gaze_overlap: float | None = None
    g_eye_left: CartSys3d | None = None
    g_eye_right: CartSys3d | None = None
    g_nose: CartSys3d | None = None
    g_head_position: CartSys3d | None = None
    g_gaze: CartSys3d | None = None


@dataclass
class SstMeta(object):
    """Available information about tracked sounds.

    Args:
        id (int): ID per separable sound source, reused.
        activity (float): Activity of signal (0 to 1).
        loc (CartSys3d): Spacial source of sound.
        is_dynamic (bool): If track is dynamic or statically defined.
    """

    id: int
    activity: float
    loc: CartSys3d
    is_dynamic: bool


@dataclass
class PerceptionData(object):
    """Class containing all infomation sent via perception socket.

    Args:
        time (int): Timestap of image taken as basis for persons
        persons (list[Person]): List of detected persons.
        sst_time_latest (int): Timestamp of audio taken as basis for sst_latest
        sst_tracks_latest (list[SstMeta]): Most current list of detected sound sources.
    """

    time: int = 0
    persons: List[Person] = field(default_factory=list)
    sst_time_latest: int = 0
    sst_tracks_latest: List[SstMeta] = field(default_factory=list)



from __future__ import annotations

import asyncio
import math
import time
from typing import Awaitable, Callable

import bitstring
import navel._data_structs as ds
from navel._async import AsyncRunner, runner_task
from navel._pyproto import klframe_pb2 as _klframe
from navel._pyproto import payload_pb2 as _payload
from navel._pyproto import perc_pb2 as _perc
from navel._pyproto import speech_pb2 as _speech
from navel._pyproto import uuid_pb2 as _uuid
from navel._socket_coms import ProtobufSocket

# Wait between read socket calls in seconds
RECV_SLEEP = 0.04

# Constants for tts activity detection
TTS_STATE_NOT_SPEAKING = "PREPARED"
TTS_PROC_FILE = "/proc/asound/APE/pcm2p/sub0/status"

# Size of bodyd text buffer
SPEECH_BUF_SIZE = 1024
SPEECH_BUF_SLOTS = 128


class MotionMessenger(AsyncRunner):
    def __init__(self, socket_name: str | None = None):
        self._socket_name = socket_name
        self._arms = None
        self._tilts = None
        super().__init__()

    def open(self):
        self._motion_sock = ProtobufSocket(self._socket_name)
        super().open()

    def close(self):
        self._motion_sock.disconnect()
        super().close()

    @runner_task
    async def look_at_cart(self, cart: ds.CartSys3d, head: float):
        """Sends a message for robot to look at a point in space.

        Args:
            cart (CartSys3d): 3d target point to look at.
            head (float): Magnitude of head movement towards point, as compared
                to eye movement, where 0 is no head movement and 1 is maximum
                head movement.

        Raises:
            SocketError: In cases where message cannot be sent via socket.
        """

        msg = _payload.LookatCart()
        msg.cart.sys = cart.sys
        msg.cart.x = cart.x * 1000
        msg.cart.y = cart.y * 1000
        msg.cart.z = cart.z * 1000
        msg.head = head

        self._motion_sock.send(msg, _klframe.LOOKAT_CART)
        await asyncio.sleep(0.5)

    @runner_task
    async def look_at_px(self, px: ds.Point2d, head: float):
        """Sends a message to robot to look at a pixel.

        Args:
            px (Point2d): 2d point on the image plane.
            head (float): Magnitude of head movement towards point, as compared
                to eye movement, where 0 is no head movement and 1 is maximum
                head movement.

        Raises:
            SocketError: In cases where message cannot be sent via socket.
        """

        msg = _payload.LookatPx()
        msg.px.x = px.x
        msg.px.y = px.y
        msg.head = head

        self._motion_sock.send(msg, _klframe.LOOKAT_PX)
        await asyncio.sleep(0.5)

    @runner_task
    async def look_at_person(self, uuid: int, head: float):
        """Sends a message setting which person robot should look at.

        Args:
            uuid (int): Unique identifier of person.
            head (float): Magnitude of head movement towards point, as compared
                to eye movement, where 0 is no head movement and 1 is maximum
                head movement.
        Raises:
            SocketError: In cases where message cannot be sent via socket.
        """

        msg = _payload.LookatPerson()
        temp = _split_uuid(uuid)
        msg.uuid.c0 = temp.c0
        msg.uuid.c1 = temp.c1
        msg.uuid.c2 = temp.c2
        msg.uuid.c3 = temp.c3
        msg.head = head

        self._motion_sock.send(msg, _klframe.LOOKAT_PERSON)
        await asyncio.sleep(0.5)

    @runner_task
    async def head_overlay(self, overlay: ds.Bryan):
        """Sends a message rotating the head from its natural position (which
            is determined by the direction of focus) using Tait-Bryan angles.

        Args:
            overlay (Bryan): Tait-Bryan angles describing offset from natural
                position.

        Raises:
            SocketError: In cases where message cannot be sent via socket.
        """

        msg = _payload.HeadOverlay()
        msg.overlay.x = overlay.x
        msg.overlay.y = overlay.y
        msg.overlay.z = overlay.z

        self._motion_sock.send(msg, _klframe.HEAD_OVERLAY)

    @runner_task
    async def head_path_limits(self, v_max: float, a_max: float, j_max: float):
        """Sends pathplanner limits for head motion to robot.

        Args:
            v_max (float): Maximum velocity.
            a_max (float): Maximum acceleration.
            j_max (float): Maximum jerk.

        Raises:
            SocketError: In cases where message cannot be sent via socket.
        """

        msg = _payload.HeadBndry()
        msg.v_max = v_max
        msg.a_max = a_max
        msg.j_max = j_max
        self._motion_sock.send(msg, _klframe.HEAD_BNDRY)

    @runner_task
    async def head_facial_expression(
        self,
        neutral: float = 0,
        happy: float = 0,
        sad: float = 0,
        surprise: float = 0,
        anger: float = 0,
        smile: float = 0,
    ):
        """Sends a message setting the facial expression of the robot.

        This function sets the magnitude of each of the robots five expressions.
        Each argument contains a scalar value of how extreme the argument
        emotion is, where emotion max is 1 and emotion minimum is 0.

        Args:
            neutral (float): Sets magnitude of neutral expression.
            happy (float): Sets magnitude of happy expression.
            sad (float): Sets magnitude of sad expression.
            surprise (float): Sets magnitude of surprise expression.
            anger (float): Sets magnitude of anger expression.
            smile (float): Sets magnitude of smile expression.

        Raises:
            SocketError: In cases where message cannot be sent via socket.
        """

        msg = _payload.HeadFe()
        msg.neutral = neutral
        msg.happy = happy
        msg.sad = sad
        msg.surprise = surprise
        msg.anger = anger
        msg.smile = smile

        self._motion_sock.send(msg, _klframe.HEAD_FE)

    @runner_task
    async def head_eyelids(self, openness_l: float, openness_r: float):
        """Sends a message setting how open the eyes should be.

        Args:
            openness_l (float): Scalar value of how open left eye should be,
                where 1 is open and 0 is closed.
            openness_r (float): Scalar value of how open right eye should be,
                where 1 is open and 0 is closed.

        Raises:
            SocketError: In cases where message cannot be sent via socket.
        """

        msg = _payload.HeadEyelids()
        msg.openness_l = openness_l
        msg.openness_r = openness_r

        self._motion_sock.send(msg, _klframe.HEAD_EYELIDS)

    @runner_task
    async def head_mouth(self, openness: float):
        """Sends a message setting how wide the mouth should be open.

        Args:
            openness (float): Scalar value which controls how wide mouth
                should be open, 1 = open, 0 = closed.

        Raises:
            SocketError: In cases where message cannot be sent via socket.
        """

        msg = _payload.HeadMouth()
        msg.openness = openness

        self._motion_sock.send(msg, _klframe.HEAD_MOUTH)

    @runner_task
    async def config_set(self, name, value, valType):
        """Sends a message to change the value of a config field.

        Args:
            name (str): Name of config field to set. Parameters can be explored
                in browser interface. Concatenate name segments from
                there, e.g. "rndr_rgbsensor_max_brightness".
            value (int or float): Value to set.
            valType (DataType): DataType Enum describing value datatype.

        Raises:
            SocketError: In cases where message cannot be sent via socket.
        """

        msg = _payload.ConfigSet()
        msg.name = name

        if valType == ds.DataType.I32:
            msg.i32 = value
        elif valType == ds.DataType.U32:
            msg.u32 = value
        elif valType == ds.DataType.I64:
            msg.i64 = value
        elif valType == ds.DataType.U64:
            msg.u64 = value
        elif valType == ds.DataType.F32:
            msg.f32 = value
        elif valType == ds.DataType.F64:
            msg.f64 = value

        else:
            raise ValueError("Unknown data type")

        self._motion_sock.send(msg, _klframe.CONFIG_SET)

    @runner_task
    async def move_base(
        self,
        distance: float,
        speed: float | None = None,
        acceleration: float | None = None,
    ):
        """Sends a move command to robot.

        Command translates the robot forward/backward by desired distance.
        If no speed/acceleration are defined, the maximum will be used.

        Args:
            distance (float): Motion magnitude and direction in meters.
            speed (float): Desired peak speed (m/s).
            acceleration (float): Desired peak acceleration (m/s²).

        Raises:
            ValueError: When speed/acceleration are outside limits.
            SocketError: When message cannot be sent via socket.
        """

        # TODO: get this from bodyd
        peak_speed = 1.6
        peak_acc = 1.2

        if speed is None:
            speed = peak_speed
        if acceleration is None:
            acceleration = peak_acc

        _check_limits(speed, 0, peak_speed, "Speed")
        _check_limits(acceleration, 0, peak_acc, "Acceleration")

        t_c, t_a = _calc_motion_times(distance, speed, acceleration)
        v = int(math.copysign(speed, distance) * 10000)  # µm / 10 ms

        await self._move(t_c, t_a, v, 0)

    @runner_task
    async def rotate_base(
        self,
        angle: float,
        speed: float | None = None,
        acceleration: float | None = None,
    ):
        """Sends a rotate command to robot.

        Command rotates the robot around Z axis by desired angle.
        If no speed/acceleration are defined, the maximum will be used.

        Args:
            angle (float): Rotation magnitude and direction in degrees.
            speed (float): Desired peak speed (deg/s).
            acceleration (float): Desired peak acceleration (deg/s²).

        Raises:
            ValueError: When speed/acceleration are outside limits.
            SocketError: When message cannot be sent via socket.
        """

        # TODO: get this from bodyd
        peak_speed = 70
        peak_acc = 60

        if speed is None:
            speed = peak_speed
        if acceleration is None:
            acceleration = peak_acc

        _check_limits(speed, 0, peak_speed, "Speed")
        _check_limits(acceleration, 0, peak_acc, "Acceleration")

        t_c, t_a = _calc_motion_times(angle, speed, acceleration)
        # µrad / 10 ms
        v = int(math.copysign(speed, angle) * 10000 / 180 * math.pi)

        await self._move(t_c, t_a, 0, v)

    @runner_task
    async def move_and_rotate_base(
        self,
        distance: float,
        angle: float,
        speed: float | None = None,
        acceleration: float | None = None,
    ):
        """Sends both a move and a rotate command to robot.

        Command translates the robot forward/backward by the desired
        distance and rotates it around Z axis by desired angle.
        If no speed/acceleration are defined, the maximum will be used.
        Rotational speed is calculated to match movement speed.

        Warning: rotation amount is likely to be inaccurate, some trial
        and error may be necessary to define wanted movements.

        Args:
            distance (float): Motion magnitude and direction in meters.
            angle (float): Rotation magnitude and direction in degrees.
            speed (float): Desired peak movement speed (m/s).
            acceleration (float): Desired peak acceleration (m/s²).

        Raises:
            ValueError: When speed/acceleration are outside limits.
            SocketError: When message cannot be sent via socket.
        """

        # TODO: get this from bodyd
        peak_speed = 0.25
        peak_acc = 1
        if speed is None:
            speed = peak_speed
        if acceleration is None:
            acceleration = peak_acc
        _check_limits(speed, 0, peak_speed, "Speed")
        _check_limits(acceleration, 0, peak_acc, "Acceleration")

        t_c, t_a = _calc_motion_times(distance, speed, acceleration)
        vl = int(math.copysign(speed, distance) * 10000)

        acc_time = t_a / 100
        const_time = t_c / 100
        spd = angle / (const_time + acc_time)  # should be the peak speed
        vr = int(math.copysign(spd, angle) * 10000 / 180 * math.pi)

        await self._move(t_c, t_a, vl, vr)

    async def _move(self, t_c, t_a, v_l, v_r):
        for i in range(t_a):
            lin = int(_ramp(i, t_a) * v_l)
            rot = int(_ramp(i, t_a) * v_r)

            self._base_move(lin, rot)
            await asyncio.sleep(0.0099)

        for i in range(t_c):  # constant vel
            self._base_move(v_l, v_r)
            await asyncio.sleep(0.0099)

        for i in reversed(range(t_a)):
            lin = int(_ramp(i, t_a) * v_l)
            rot = int(_ramp(i, t_a) * v_r)

            self._base_move(lin, rot)
            await asyncio.sleep(0.0099)

    @runner_task
    async def rotate_arms(self, left_angle: float, right_angle: float):
        """Sends rotate arms command to robot.

        The arm motion is limited in the upper back and arms can not do a full
        rotation. Rotation is around robot's -Y axis, 0 is straight down,
        positive is generally up and negative is generally down.

        Args:
            angle_l (float): Absolute rotation of left arm in degrees.
            angle_r (float): Absolute rotation of right arm, in degrees.

        Raises:
            SocketError: In cases where message cannot be sent via socket.
        """

        # TODO: move to bodyd
        factor = 3100000 / 180

        msg = _payload.Arms()
        msg.angle_l = int(left_angle * factor)
        msg.angle_r = int(-right_angle * factor)

        self._motion_sock.send(msg, _klframe.ARMS)

        curr = (left_angle, right_angle)
        d = 245 if self._arms is None else _max_diff(curr, self._arms)
        self._arms = curr
        await asyncio.sleep(0.5 + d / 180 * 2)

    @runner_task
    async def tilt_base(self, x_percent: float, y_percent: float):
        """Sends tilt body command to robot.

        Args:
            x (float): Tilt around X axis as a fraction of maximum.
                -1 (left) to 1 (right).
            y (float): Tilt around Y axis as a fraction of maximum.
                -1 (backwards) to 1 (forwards).

        Raises:
            SocketError: In cases where message cannot be sent via socket.
        """

        center_x = 0
        amp_x = 1000
        center_y = 0
        amp_y = 1000

        msg = _payload.Basetilt()
        msg.x = int(x_percent * amp_x + center_x)
        msg.y = int(y_percent * amp_y + center_y)

        self._motion_sock.send(msg, _klframe.BASETILT)

        curr = (x_percent, y_percent)
        d = 2 if self._tilts is None else _max_diff(curr, self._tilts)
        self._tilts = curr
        await asyncio.sleep(0.5 + d / 2)

    def _base_move(self, x, r):
        """Sends a move command to robot.

        Command sets the robot speed to desired value, if after 10ms no new
        base_move has been sent, then robot will start to decrease velocity.
        To keep robot at same velocity, issue this command with the desired speed
        every 10ms.

        Args:
            x (int): Forward motion magnitude. Send 1000 every 10ms to go with
                1000µm/10ms = 0.1m/s.
            r (int): Anticlockwise rotation (around Z). Send 1000 every 10ms to
                go with 1000µrad/10ms = 0.1rad/s.

        Raises:
            SocketError: In cases where message cannot be sent via socket.
        """

        msg = _payload.Baseinc()
        msg.x = x
        msg.y = 0
        msg.z = r

        self._motion_sock.send(msg, _klframe.BASEINC)


def _check_limits(val, minimum, maximum, name):
    if val > maximum:
        raise ValueError(f"{name} above allowed maximum ({maximum}).")
    if val < minimum:
        raise ValueError(f"{name} below allowed minimum ({minimum}).")


def _calc_motion_times(dist, speed, acceleration):
    if speed == 0 or acceleration == 0 or dist == 0:
        return 0, 0

    acc_time = speed / acceleration

    # Make sure we don't go too far while accelerating to target speed

    while True:
        c_dist = abs(dist) - acc_time * speed  # symmetric acc/dec
        if c_dist >= 0:
            break
        acc_time -= 0.01  # minimum time step

    t_c = int(c_dist / speed * 100)  # constant travel time in 10ms
    t_a = int(acc_time * 100)  # acc/brake time in 10ms

    return t_c, t_a


def _ramp(i, m):
    """sine ramp: 0 to m -> 0 to 0.5"""
    return (-math.cos(math.pi * i / m) + 1) / 2


def _max_diff(curr, prev):
    return max((abs(curr[0] - prev[0]), abs(curr[1] - prev[1])))


class SpeechMessenger(AsyncRunner):
    """Class containing methods to access speech messages.

    This class contains methods to send and receive speech messages. As robot
    also returns messages on this socket this class inherits AsyncSocketListener
    to asynchronously detect such messages. Incoming messages are read by
    internal message handler.
    """

    def __init__(self, socket_name: str | None = None):
        self._socket_name = socket_name
        self._bookmark_cbs = {}
        super().__init__()

    def open(self):
        self._speech_sock = ProtobufSocket(self._socket_name)
        super().open()

    def close(self):
        self._speech_sock.disconnect()
        super().close()

    async def _handle_bookmark(self, num):
        if num in self._bookmark_cbs:
            asyncio.create_task(self._bookmark_cbs[num](self))

    @runner_task
    async def say(self, text: str):
        """Sends a speech text message to speech socket.
        If the message is too long, it may be split up
        and sent as parts.

        Args:
            text (str): contents of message to be sent to speech socket.

        Raises:
            TypeError: If text is not string
            SocketError: If errors connecting to socket or sending message
        """

        parts = _split_text_recursive([text])

        for i in range(len(parts) // SPEECH_BUF_SLOTS + 1):
            for part in parts[
                SPEECH_BUF_SLOTS * i : SPEECH_BUF_SLOTS * (i + 1)
            ]:
                self._say_one(part)

            # Small wait to let everything start properly
            # Otherwise we get glitches in is_speaking()
            await asyncio.sleep(RECV_SLEEP)

            # Wait until we start speaking
            while not is_speaking():
                await asyncio.sleep(RECV_SLEEP)

            # Wait until we're done speaking
            while is_speaking():
                try:
                    msg = self._speech_sock.recv()
                except BlockingIOError:
                    await asyncio.sleep(RECV_SLEEP)
                    continue

                if msg.ptype == _klframe.SPEECH_BOOKMARK:
                    await self._handle_bookmark(_parse_bookmark_msg(msg))

    def _say_one(self, text: str):

        msg = _speech.SpeechText()
        msg.speech = text

        self._speech_sock.send(msg, _klframe.SPEECH_TEXT)

    def set_bookmark_callback(
        self, bookmark: int, action: Callable[[SpeechMessenger], Awaitable]
    ) -> None:
        """Sets a callback for the given bookmark number.

        Args:
            bookmark (int): Bookmark number to bind this action to.

            action (callable): Function that should be called.
                Must be async and take a reference to this object
                as its only parameter
        """

        self._bookmark_cbs[bookmark] = action


def _split_text_recursive(
    parts: list[str], delimiters: list[str] = ("\n\n", "\n", ".", ",", " ")
) -> list[str]:
    ret = []
    for part in parts:
        if part and len(part) < SPEECH_BUF_SIZE:
            ret.append(part)
            continue

        split_part = [p + delimiters[0] for p in part.split(delimiters[0])]

        if len(delimiters) > 1:
            split_part = _split_text_recursive(split_part, delimiters[1:])
        else:
            # If no delimiters left, just split into sections of the right size
            _ls = []
            for p in split_part:
                _ls.extend(
                    [
                        p[SPEECH_BUF_SIZE * i : SPEECH_BUF_SIZE * (i + 1)]
                        for i in range(len(p) // SPEECH_BUF_SIZE)
                    ]
                )

            split_part = _ls

        ret.extend(split_part)

    ret = _combine_text(ret)
    return ret


def _combine_text(parts):
    combined = []
    current = ""
    for idx, p in enumerate(parts):
        nxt = current + p
        if len(nxt) >= SPEECH_BUF_SIZE:
            combined.append(current)
            current = p
            if idx == len(parts) - 1:
                combined.append(p)
        elif idx == len(parts) - 1:
            combined.append(nxt)
        else:
            current = nxt

    return combined


def is_speaking():
    # Extract current state from first line in proc file
    with open(TTS_PROC_FILE, "r") as fp:
        for line in fp:
            if "state:" in line:
                state = line.split()[1]

                # Differnt states can mean speaking, only one means silence
                return state != TTS_STATE_NOT_SPEAKING

    return False


class PerceptionMessenger(AsyncRunner):
    """Class for accessing perception messages."""

    def __init__(self, socket_name: str | None = None):
        self._socket_name = socket_name
        super().__init__()

    def open(self):
        self._perc_sock = ProtobufSocket(self._socket_name)
        super().open()

    def close(self):
        self._perc_sock.disconnect()
        super().close()

    @runner_task
    async def next_frame(
        self, timeout: float = 1
    ) -> Awaitable[ds.PerceptionData]:
        """Receives next frame of perception data from the robot.

        Will always return the newest available frame, which may be from
        slightly before the function was called, but never a duplicate.
        In general, frames should arrive roughly every 0.1s.

        Args:
            timeout (float, optional): Maximum time to wait for a frame
                in seconds. Defaults to 1.

        Raises:
            ConnectionAbortedError: If socket connection is lost
            TimeoutError: If timeout is triggered.

        Returns:
            Awaitable[PerceptionData]: The received perception data.
        """
        start = time.time()
        match = None
        while time.time() - start < timeout:
            try:
                msg = self._perc_sock.recv()
            except BlockingIOError:
                if match is not None:
                    return _parse_perception_msg(match)
                await asyncio.sleep(RECV_SLEEP)
                continue

            if msg is None:  # Occurs if socket connection lost
                raise ConnectionAbortedError("Socket connection lost")

            if msg.ptype == _klframe.PERC:
                match = msg

        raise TimeoutError()


def _parse_asr_duration(msg):
    asr_state = _speech.SpeechAsrState()
    asr_state.ParseFromString(msg.data)

    return asr_state.speaker_duration


def _parse_bookmark_msg(msg):
    bookmark = _speech.SpeechBookmark()
    bookmark.ParseFromString(msg.data)

    return bookmark.bookmark


# Following functions covert from protobuf to api versions of data classes.
# This is done for two reasons, firstly to improve documentation and names,
# and secondly to hide some protobuf methods and fields from external users.


def _parse_perception_msg(msg) -> ds.PerceptionData:
    data = _perc.PercData()
    data.ParseFromString(msg.data)

    flags = bitstring.BitArray(uint=data.flags, length=32)
    flags.reverse()

    # Flags is proven to now be in proper order, PERC_FLAG_FACES leftmost,
    # accessible with flags.all(True, [0]) as expected
    # length should be same as perc.h struct perc_data.flags bit length

    output = ds.PerceptionData()

    output.time = data.time

    for person in data.pers:
        per_out = ds.Person()

        per_out.uuid = _combine_uuid(person.uuid)

        # Check what fields are valid
        if flags.any(True, [0]):  # PERC_FLAG_FACES
            per_out.face = _convert_bbox_2ds(person.face)
            per_out.dist_mm = person.dist_mm

        if flags.any(True, [1]):  # PERC_FLAG_LM
            per_out.landmarks = _convert_landmarks(person.lm)

        if flags.any(True, [2]):  # PERC_FLAG_HP
            per_out.head_position = _convert_bryan(person.hp)

        if flags.any(True, [3]):  # PERC_FLAG_FER
            per_out.facial_expression = _convert_fer(person.fer)

        if flags.any(True, [4]):  # PERC_FLAG_FACEID
            per_out.id_score = person.id_score

        if flags.any(True, [5]):  # PERC_FLAG_GAZE
            per_out.gaze = _convert_cart_vec_3d(person.gaze)
            per_out.gaze_overlap = person.gaze_overlap

        if flags.any(True, [6]):  # PERC_FLAG_REF_HP
            _convert_ref_hp(person, per_out)

        output.persons.append(per_out)

    output.sst_time_latest = data.sst_time_latest

    for track_in in data.sst_track_latest:
        track = ds.SstMeta(
            track_in.id,
            track_in.activity,
            _convert_cart_sys_3d(track_in.loc),
            track_in.is_dynamic,
        )
        output.sst_tracks_latest.append(track)

    return output


def _combine_uuid(uuid):
    result = uuid.c0
    result += uuid.c1 << 32
    result += uuid.c2 << 64
    result += uuid.c3 << 96

    return result


def _split_uuid(id):
    result = _uuid.Uuid()
    result.c0 = 0xFFFFFFFF & id
    result.c1 = 0xFFFFFFFF & (id >> 32)
    result.c2 = 0xFFFFFFFF & (id >> 64)
    result.c3 = 0xFFFFFFFF & (id >> 96)

    return result


def _convert_ref_hp(input, output):
    output.g_eye_left = _convert_coord_system_list(input.g_eye_l)
    output.g_eye_right = _convert_coord_system_list(input.g_eye_r)
    output.g_nose = _convert_coord_system_list(input.g_nose)
    output.g_head_position = _convert_coord_system_list(input.g_hp)
    output.g_gaze = _convert_coord_system_list(input.g_gaze)

    return output


def _convert_coord_system_list(input):
    if input is None:
        return None

    if input[0].sys == 0:
        return None

    output = []

    for i in range(len(input)):
        output.append(_convert_cart_sys_3d(input[i]))

    return output


def _convert_coord_system(input):
    output = ds.CoordSystem(input)

    return output


def _convert_cart_vec_3d(input):
    output = ds.CartVec3d(
        input.x,
        input.y,
        input.z,
    )

    return output


def _convert_bbox_2ds(input):
    output = ds.Bbox2d(
        input.x1,
        input.y1,
        input.x2,
        input.y2,
    )

    return output


def _convert_point_2D(input):
    output = ds.Point2d(
        input.x,
        input.y,
    )

    return output


def _convert_bryan(input):
    output = ds.Bryan(
        input.x,
        input.y,
        input.z,
    )

    return output


def _convert_cart_sys_3d(input):
    output = ds.CartSys3d(
        _convert_coord_system(input.sys),
        input.x / 1000,
        input.y / 1000,
        input.z / 1000,
    )

    return output


def _convert_landmarks(input):
    output = ds.PersonLandmarks(
        _convert_point_2D(input.eye_l),
        _convert_point_2D(input.eye_r),
        _convert_point_2D(input.nose),
        _convert_point_2D(input.mouth_l),
        _convert_point_2D(input.mouth_r),
    )

    return output


def _convert_fer(input):
    output = ds.PersonFacialExpression(
        input.neutral,
        input.happy,
        input.sad,
        input.surprise,
        input.anger,
    )

    return output



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: speech.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import builder as _builder
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x0cspeech.proto\"\x1c\n\nSpeechText\x12\x0e\n\x06speech\x18\x01 \x01(\t\"*\n\x0eSpeechAsrState\x12\x18\n\x10speaker_duration\x18\x01 \x01(\r\"\"\n\x0eSpeechBookmark\x12\x10\n\x08\x62ookmark\x18\x01 \x01(\rB\nZ\x08./;protob\x06proto3')

_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, globals())
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'speech_pb2', globals())
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'Z\010./;proto'
  _SPEECHTEXT._serialized_start=16
  _SPEECHTEXT._serialized_end=44
  _SPEECHASRSTATE._serialized_start=46
  _SPEECHASRSTATE._serialized_end=88
  _SPEECHBOOKMARK._serialized_start=90
  _SPEECHBOOKMARK._serialized_end=124
# @@protoc_insertion_point(module_scope)



import asyncio

import navel


class MockClient:
    executed = False

    @navel.task
    async def mock_task(self):
        self.executed = True
        return True


def test_sync_execution():
    client = MockClient()

    res = client.mock_task()

    assert client.executed
    assert res


def test_async_execution():
    client = MockClient()

    async def run_task(c):
        c.mock_task()

    asyncio.run(run_task(client))

    assert client.executed


def test_is_task_and_returns():
    client = MockClient()
    is_task = False
    res = False

    async def run_task(c):
        nonlocal is_task
        nonlocal res
        tsk = c.mock_task()
        is_task = isinstance(tsk, asyncio.Task)
        res = await tsk

    asyncio.run(run_task(client))

    assert is_task
    assert res



import asyncio
from functools import wraps
from warnings import warn


class AsyncRunner:
    def __init__(self):
        self._active = False
        self._tasks = None

    def __enter__(self):
        if is_async():
            raise RuntimeError(
                'An event loop is running. Did you mean to use "async with"?'
            )
        self._tasks = None

        self.open()

        return self

    async def __aenter__(self):
        self._tasks = set()

        self.open()

        return self

    def open(self):
        self._active = True

    def __exit__(self, a, b, c):
        self.close()

    async def __aexit__(self, a, b, c):
        while self._tasks:
            await self._tasks.pop()

        self.close()

    def close(self):
        self._active = False

    def run(self, app):
        warn(
            "Function .run() is deprecated and will be removed in a future version. Please check the getting started guide for the recommended way to start coroutines.",
            DeprecationWarning,
            stacklevel=2,
        )
        if self._active:
            return asyncio.run(app(self))
        else:
            raise RuntimeError('App must be run inside a "with" block.')


def runner_task(func):
    """Wrapper to make coroutines "runnable".

    Returns a task if event loop is running, otherwise runs it synchronously.
    """

    @wraps(func)
    def wrapped(self: AsyncRunner, *args, **kwargs):

        if self._tasks is None:
            if is_async():
                raise RuntimeError("Sync function called from async context.")
            return asyncio.run(func(self, *args, **kwargs))
        else:
            if not is_async():
                raise RuntimeError("Async function called from sync context.")
            _task = asyncio.create_task(func(self, *args, **kwargs))
            self._tasks.add(_task)
            _task.add_done_callback(self._tasks.discard)

            return _task

    return wrapped


def is_async():
    try:
        asyncio.get_running_loop()
    except RuntimeError:
        return False

    return True



import navel
import pytest


def test_robot_run():
    async def mock_app(robot):
        return isinstance(robot, navel.Robot)

    with navel.Robot() as rob:
        res = rob.run(mock_app)

    assert res


def test_run_raises_outside_context():
    async def mock_app(robot):
        pass

    rob = navel.Robot()

    with pytest.raises(RuntimeError):
        rob.run(mock_app)


def test_navel_run():
    async def mock_app(robot):
        return isinstance(robot, navel.Robot)

    res = navel.run(mock_app)

    assert res
